package cscie259.project4.scamazon;

import java.io.IOException;
import java.io.PrintWriter;

import java.util.Enumeration;

import javax.servlet.ServletException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import javax.xml.transform.ErrorListener;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;

import javax.xml.transform.sax.SAXSource;

import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import org.xml.sax.helpers.XMLReaderFactory;


/**
 * This servlet implements Scamazon.com's catalog.
 * <p/>
 * Any references to the local file system are resolved
 * with respect to project4-6.0/webapps/scamazon/, per
 * ScamazonServlet's implementation of javax.xml.transform.URIResolver.
 * <p/>
 * You MAY modify this file.
 *
 * @author  Computer Science E-259
 * @version 6.0
 *
 * @author  YOUR NAME GOES HERE
 **/

public class Catalog extends ScamazonServlet
{
    /**
     * Handles operations involving the catalog.
     *
     * @param request HTTP request object
     * @param response HTTP response object
     *
     * @throws IOException if an IO-related error occurs
     * @throws ServletException if a servlet-related error occurs
     **/

    public void doWork(HttpServletRequest request, 
                       HttpServletResponse response)
    throws IOException, ServletException
    {
        // output text/html MIME type
        response.setContentType("text/html");

        // retrieve PrintWriter
        PrintWriter out = response.getWriter();

        // attempt to process catalog operation, passing any parameters
        // to stylesheet
        try
        {
            // instantiate a transformer factory
            TransformerFactory tFactory = TransformerFactory.newInstance();

            // instantiate AnErrorListener
            AnErrorListener errorListener = new AnErrorListener();

            // handle errors during processing of transformation instructions
            tFactory.setErrorListener(errorListener);

            // configure URI resolving
            tFactory.setURIResolver(new myURIResolver());

            // load stylesheet
            StreamSource xslSource = new StreamSource(getServletContext().
                                     getResourceAsStream("/xsl/catalog.xsl"));

            // instantiate transformer
            Transformer transformer = tFactory.newTransformer(xslSource);

            // handle errors during the transformation
            transformer.setErrorListener(errorListener);

            // pass HTTP parameters to stylesheet
            Enumeration names = request.getParameterNames();
            while (names.hasMoreElements())
            {
                String name = (String)names.nextElement();
                String value = request.getParameter(name);
                transformer.setParameter(name, value);
            }

            // create an XMLReader
            XMLReader reader
             = XMLReaderFactory.createXMLReader();

            // enable validation as appropriate
            String validation = System.getProperty("enable.validation");
            if (validation != null && validation.equals("dtd"))
            {
                reader.setFeature("http://xml.org/sax/features/validation",
                                  true);
			}
            else if (validation != null && validation.equals("xsd"))
            {
                reader.setFeature("http://xml.org/sax/features/validation",
                                  true);
                reader.setFeature("http://apache.org/xml/features/validation/schema",
                                  true);
            }
            else
            {
                reader.setFeature("http://xml.org/sax/features/validation",
                                  false);
			}

            // load XML
            SAXSource saxSource = new SAXSource(reader,
              new InputSource(getServletContext().
               getResourceAsStream("/xml/catalog.xml")));

            // transform!
            transformer.transform(saxSource, new StreamResult(out));

        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw new ServletException("Can't process XSLT: " + e.getMessage());
        }
    }
}
