package cscie259.project4.scamazon;

import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerException;
import javax.xml.transform.Source;
import javax.xml.transform.Result;
import javax.xml.transform.URIResolver;

import javax.xml.transform.sax.SAXResult;

import javax.xml.transform.stream.StreamSource;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import org.apache.fop.apps.Driver;
import org.apache.fop.apps.Version;
import org.apache.fop.apps.XSLTInputHandler;
import org.apache.fop.messaging.MessageHandler;

import org.apache.avalon.framework.logger.ConsoleLogger;
import org.apache.avalon.framework.logger.Logger;


/**
 * This servlet produces Scamazon.com's receipts as PDFs.
 * <p/>
 * Any references to the local file system in the servlet's context 
 * are resolved with respect to project4-6.0/webapps/scamazon/.
 * <p/>
 * Based on version 1.1.2.1 of Apache's FopServlet.java,
 * per the below copyright notice; original source from
 * FOP 0.20.5rc2's distribution.
 * <p/>
 *   $Id: FopServlet.java,v 1.1.2.1 2003/01/13 16:27:14 jeremias Exp $
 *   Copyright (C) 2001 The Apache Software Foundation. All rights reserved.
 *   For details on use and redistribution please refer to the
 *   LICENSE file included with these sources.
 * <p/>
 * Given an XSL-FO file or, alternatively, XML and XSLT which together
 * yield XSL-FO, produces PDF, which is returned directly to the 
 * browser.
 * <p/>
 * Current usage:
 * <pre>
 *   http://$HOSTNAME.fas.harvard.edu:n/scamazon/servlet/receipt?fo=foo.fo&ext=.pdf
 *   http://$HOSTNAME.fas.harvard.edu:n/scamazon/servlet/receipt?xml=foo.xml&xsl=foo.xsl&ext=.pdf
 * </pre>
 * Per http://xml.apache.org/fop/embedding.html, "Some browsers have problems 
 * handling the PDF result sent back to the browser. IE is particularly bad 
 * and different versions behave differently.  Having a '.pdf' on the end of 
 * the URL may help."
 * <p/>
 * Hence our inclusion of the unused "ext" parameter in the current usage.
 * <p/>
 * You MAY modify this file.
 *
 * @author  The Apache Software Foundation
 * @author  Computer Science E-259
 * @version 6.0
 *
 * @author  YOUR NAME GOES HERE
 **/

public class Receipt extends ScamazonServlet 
{
    /**
     * HTTP parameter referencing an XSL-FO file
     */
    public static final String FO_REQUEST_PARAM = "fo";


    /**
     * HTTP parameter referencing an XML file
     */
    public static final String XML_REQUEST_PARAM = "xml";


    /**
     * HTTP parameter referencing an XSLT file
     */
    public static final String XSLT_REQUEST_PARAM = "xsl";


    /* reference to a Logger */
    private Logger log = null;


    /**
     * Responds to GETs in the same manner as POSTs.
     *
     * @param request HTTP request object
     * @param response HTTP response object
     *
     * @throws IOException if an IO-related error occurs
     * @throws ServletException if a servlet-related error occurs
     **/
    
    public void doGet(HttpServletRequest request, 
                      HttpServletResponse response)
    throws IOException, ServletException
    {   
        doWork(request, response);
    }

    
    /**
     * Respond to POSTs in the same manner as GETs.
     *
     * @param request HTTP request object
     * @param response HTTP response object
     *
     * @throws IOException if an IO-related error occurs
     * @throws ServletException if a servlet-related error occurs
     **/
    public void doPost(HttpServletRequest request, 
                       HttpServletResponse response)
    throws IOException, ServletException
    {   
        doWork(request, response);
    }


    /**
     * Handles operations involving receipts.
     *
     * @param request HTTP request object
     * @param response HTTP response object
     *
     * @throws ServletException if a servlet-related error occurs
     **/
    public void doWork(HttpServletRequest request, 
                       HttpServletResponse response)
    throws ServletException 
    {
        // instantiate Logger as necessary
        if (log == null) 
        {
            log = new ConsoleLogger(ConsoleLogger.LEVEL_WARN);
            MessageHandler.setScreenLogger(log);
        }

        // attempt to process request
        try 
        {
            // retrieve HTTP parameters
            String foParam  = request.getParameter(FO_REQUEST_PARAM);
            String xmlParam = request.getParameter(XML_REQUEST_PARAM);
            String xslParam = request.getParameter(XSLT_REQUEST_PARAM);

            // process XSL-FO request; assume (arbitrarily) that file is in 
            // project4-6.0/webapps/scamazon/fo/
            if (foParam != null) 
            {
                InputSource inputSource 
                = new InputSource(getServletContext().
                                  getResourceAsStream("/fo/" + foParam));
                renderFO(inputSource, response);
            } 

            // process XML/XSLT request
            else if ((xmlParam != null) && (xslParam != null)) 
            {
                processXSL(xmlParam, xslParam, response);
            } 

            // err
            else 
            {
                // output text/plain MIME type
                response.setContentType("text/plain");

                // retrieve PrintWriter
                PrintWriter out = response.getWriter();

                // report error
                out.println("Error: improper usage.");
            }
        } 
        catch (ServletException e) 
        {
            throw e;
        }
        catch (Exception e) 
        {
            throw new ServletException(e);
        }
    }


    /**
     * Renders an FO input source into a PDF file which is written
     * directly to the response object's OutputStream.
     *
     * @param foFile    XSL-FO file
     * @param response  HTTP response object
     *
     * @throws ServletException if a servlet-related error occurs
     */
    public void renderFO(InputSource foFile, HttpServletResponse response) 
    throws ServletException 
    {
        // attempt to generate receipt
        try 
        {
            // instantiate new output stream
            ByteArrayOutputStream out = new ByteArrayOutputStream();

            // output application/pdf MIME type
            response.setContentType("application/pdf");

            // process XSL-FO, storing results in output stream
            Driver driver = new Driver(foFile, out);
            driver.setLogger(log);
            driver.setRenderer(Driver.RENDER_PDF);
            driver.run();

            // return output
            byte [] content = out.toByteArray();
            response.setContentLength(content.length);
            response.getOutputStream().write(content);
            response.getOutputStream().flush();
        } 
        catch (Exception e) 
        {
            throw new ServletException(e);
        }
    }


    /**
     * Renders an XML file into a PDF file by applying a stylesheet
     * that converts the XML to XSL-FO.  The PDF is written
     * directly to the response object's OutputStream.
     *
     * @param xml       name of XML input file
     * @param xsl       name of XSLT input file
     * @param response  HTTP response object
     *
     * @throws ServletException if a servlet-related error occurs
     */
    public void processXSL(String xml, String xsl, 
                           HttpServletResponse response)
    throws ServletException 
    {
        // instantiate driver
        Driver driver = new Driver();

        // instantiate and configure Logger
        Logger logger = new ConsoleLogger(ConsoleLogger.LEVEL_INFO);
        driver.setLogger(logger);
        MessageHandler.setScreenLogger(logger);
    
        // configure renderer
        driver.setRenderer(Driver.RENDER_PDF);

        // instantiate and configure output stream
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        driver.setOutputStream(out);

        // output application/pdf MIME type
        response.setContentType("application/pdf");

        // attempt to generate receipt
        try 
        {
            // instantiate a transformer factory
            TransformerFactory tFactory = TransformerFactory.newInstance();

            // configure URI resolving
            tFactory.setURIResolver(new myURIResolver());

            // load stylesheet; assume (arbitrarily) that file is in 
            // project4-6.0/webapps/scamazon/xsl/
            StreamSource xslSource 
            = new StreamSource(getServletContext().
                               getResourceAsStream("/xsl/" + xsl));
    
            // instantiate transformer
            Transformer transformer = tFactory.newTransformer(xslSource);

            // resulting SAX events (the generated XSL-FO) 
            // must be piped through to FOP
            Result res = new SAXResult(driver.getContentHandler());

            // load XML; assume (arbitrarily) that file is in 
            // project4-6.0/webapps/scamazon/xml/
            Source xmlSource 
             = new StreamSource(getServletContext().
                                getResourceAsStream("/xml/" + xml));

            // start XSLT transformation and FOP processing
            transformer.transform(xmlSource, res);

            // return output
            byte [] content = out.toByteArray();
            response.setContentLength(content.length);
            response.getOutputStream().write(content);
            response.getOutputStream().flush();
        }
        catch (Exception e) 
        {
            throw new ServletException(e);
        }
    }


    /**
     * Resolves references to files.
     **/
    private class myURIResolver implements URIResolver
    {
        /**
         * Resolves references to files with respect to
         * project4-6.0/webapps/scamazon/.
         *
         * @param href  path to file
         * @param base  current base
         *
         * @throws TransformerException if a transformation-related error occurs
         **/
        public Source resolve(String href, String base)
        throws TransformerException
        {
            return (new StreamSource(getServletContext().
                                     getResourceAsStream("/" + href)));
        }
    }
}
